@@ -8,27 +8,30 @@
  ************************************************************************/
 #ifdef RCS
 static /*const*/char rcsid[]=
- "$Id: exopen.c,v 1.37.2.4 2001/07/15 09:27:13 guenther Exp $";
+ "$Id: exopen.c,v 1.42 2001/06/27 06:41:25 guenther Exp $";
 #endif
 #include "procmail.h"
 #include "acommon.h"
 #include "robust.h"
 #include "misc.h"
-#include "common.h"
 #include "exopen.h"
 #include "lastdirsep.h"
+#include "sublib.h"
 
-int unique(full,p,len,mode,verbos,chownit)char*const full;char*p;
- const size_t len;const mode_t mode;const int verbos,chownit;
+int unique(full,p,len,mode,verbos,flags)char*const full;char*p;
+ const size_t len;const mode_t mode;const int verbos,flags;
 { static const char s2c[]=".,+%";static int serial=STRLEN(s2c);
-  static time_t t;char*dot,*end=full+len,*op,*ldp;struct stat filebuf;
+  static time_t t;char*dot,*end,*op,*ldp;struct stat filebuf;
   int nicediff,i,didnice,retry=RETRYunique;
-  if(chownit&doCHOWN)		  /* semi-critical, try raising the priority */
+  if(flags&doCHOWN)		  /* semi-critical, try raising the priority */
    { nicediff=nice(0);SETerrno(0);nicediff-=nice(-NICE_RANGE);
      didnice=!errno;
    }
-  *(end=len?full+len-1:p+UNIQnamelen-1)='\0';
-  *(op=p)=UNIQ_PREFIX;dot=ultoan((long)thepid,p+1);
+  *(end=len?full+len-1:(op=p)+UNIQnamelen-1)='\0';
+  if(flags&doMAILDIR)				/* 'official' maildir format */
+     dot=p;
+  else						     /* 'traditional' format */
+     *p=UNIQ_PREFIX,dot=ultoan((long)thepid,p+1);
   if(serial<STRLEN(s2c))
      goto in;
   do
@@ -38,11 +41,22 @@ int unique(full,p,len,mode,verbos,chownit)char*const full;char*p;
 	      ssleep(1);				   /* tap tap tap... */
 	   serial=0;t=t2;
 	 }
-in:	p=ultoan((long)t,dot+1);
-	*p++='.';
-	strncpy(p,hostname(),end-p);
+in:	if(flags&doMAILDIR)
+	 { ultstr(0,(long)t,p);
+	   *(dot=strchr(p,'\0'))='.';
+	   ultstr(0,(long)thepid,dot+1);
+	   *(dot=strchr(p,'\0'))='_';
+	   *(++dot+1)='.';
+	   strlcat(dot+2,hostname(),end-dot);
+	 }
+	else
+	 { p=ultoan((long)t,dot+1);
+	   *p++='.';
+	   strncpy(p,hostname(),end-p);
+	 }
       }
-     *dot=s2c[serial++];
+     *dot=(flags&doMAILDIR)?'0'+serial:s2c[serial];
+     serial++;
      i=lstat(full,&filebuf);
 #ifdef ENAMETOOLONG
      if(i&&errno==ENAMETOOLONG)
@@ -65,7 +79,7 @@ in:	p=ultoan((long)t,dot+1);
   while((!i||errno!=ENOENT||	      /* casually check if it already exists */
 	 (0>(i=ropen(full,O_WRONLY|O_CREAT|O_EXCL,mode))&&errno==EEXIST))&&
 	(i= -1,retry--));
-  if(chownit&doCHOWN&&didnice)
+  if(flags&doCHOWN&&didnice)
      nice(nicediff);		   /* put back the priority to the old level */
   if(i<0)
    { if(verbos)			      /* this error message can be confusing */
@@ -73,25 +87,25 @@ in:	p=ultoan((long)t,dot+1);
      goto ret0;
    }
 #ifdef NOfstat
-  if(chownit&doCHOWN)
+  if(flags&doCHOWN)
    { if(
 #else
-  if(chownit&doCHECK)
+  if(flags&doCHECK)
    { struct stat fdbuf;
      fstat(i,&fdbuf);			/* match between the file descriptor */
 #define NEQ(what)	(fdbuf.what!=filebuf.what)	    /* and the file? */
      if(lstat(full,&filebuf)||filebuf.st_nlink!=1||filebuf.st_size||
 	NEQ(st_dev)||NEQ(st_ino)||NEQ(st_uid)||NEQ(st_gid)||
-	 chownit&doCHOWN&&
+	 flags&doCHOWN&&
 #endif
 	 chown(full,uid,sgid))
       { rclose(i);unlink(full);			 /* forget it, no permission */
-ret0:	return chownit&doFD?-1:0;
+ret0:	return flags&doFD?-1:0;
       }
    }
-  if(chownit&doLOCK)
+  if(flags&doLOCK)
      rwrite(i,"0",1);			   /* pid 0, `works' across networks */
-  if(chownit&doFD)
+  if(flags&doFD)
      return i;
   rclose(i);
   return 1;
