@@ -8,24 +8,22 @@
  ************************************************************************/
 #ifdef RCS
 static /*const*/char rcsid[]=
- "$Id: pipes.c,v 1.62.2.2 2001/07/15 09:27:27 guenther Exp $";
+ "$Id: pipes.c,v 1.71 2001/06/23 08:18:49 guenther Exp $";
 #endif
 #include "procmail.h"
 #include "robust.h"
 #include "shell.h"
 #include "misc.h"
+#include "memblk.h"
 #include "pipes.h"
 #include "common.h"
-#include "cstdio.h"
-#include "exopen.h"
 #include "mcommon.h"
-#include "goodies.h"
 #include "foldinfo.h"
 #include "mailfold.h"
+#include "goodies.h"
+#include "variables.h"
 
-const char exitcode[]="EXITCODE";
 static const char comma[]=",";
-int setxit;
 pid_t pidchild;
 volatile time_t alrmtime;
 volatile int toutflag;
@@ -35,6 +33,13 @@ static pid_t pidfilt;
 static int pbackfd[2];			       /* the emergency backpipe :-) */
 int pipw;
 
+#define PRDO	poutfd[0]
+#define PWRO	poutfd[1]
+#define PRDI	pinfd[0]
+#define PWRI	pinfd[1]
+#define PRDB	pbackfd[0]
+#define PWRB	pbackfd[1]
+
 void inittmout(progname)const char*const progname;
 { lastexec=cstr(lastexec,progname);toutflag=0;
   alrmtime=timeoutv?time((time_t*)0)+(unsigned)timeoutv:0;
@@ -78,7 +83,7 @@ static void getstdin(pip)const int pip;
 static void callnewprog(newname)const char*const newname;
 {
 #ifdef RESTRICT_EXEC
-  if(mailfilter!=2&&erestrict&&uid>=RESTRICT_EXEC)
+  if(erestrict&&uid>=RESTRICT_EXEC)
    { syslog(LOG_ERR,slogstr,"Attempt to execute",newname);
      nlog("No permission to execute");logqnl(newname);
      return;
@@ -178,9 +183,14 @@ perr:	      progerr(line,excode,pwait==4);  /* I'm going to tell my mommy! */
   if(forkerr(pidchild,procmailn))
      return -1;
   if(Stdout)
-   { char*name;
+   { char*name;memblk temp;		    /* ick.  This is inefficient XXX */
      *eq='=';name=Stdout;Stdout=0;primeStdout(name);free(name);
-     Stdout=readdyn(Stdout,&Stdfilled);
+     makeblock(&temp,Stdfilled);
+     tmemmove(temp.p,Stdout,Stdfilled);
+     readdyn(&temp,&Stdfilled,0);
+     Stdout=realloc(Stdout,&Stdfilled+1);
+     tmemmove(Stdout,temp.p,Stdfilled+1);
+     freeblock(&temp);
      retStdout(Stdout,pwait&&pipw,!backblock);
      return pipw;
    }
@@ -188,7 +198,7 @@ perr:	      progerr(line,excode,pwait==4);  /* I'm going to tell my mommy! */
 }
 
 long pipin(line,source,len,asgnlastf)char*const line;char*source;long len;
- const int asgnlastf;
+ int asgnlastf;
 { int poutfd[2];
 #if 0						     /* not supported (yet?) */
   if(!sh)					/* shortcut builtin commands */
@@ -246,47 +256,38 @@ builtin:
   return len;
 }
 
-char*readdyn(bf,filled)char*bf;long*const filled;
-{ int blksiz=BLKSIZ;long oldsize= *filled;unsigned shift=EXPBLKSIZ;char*np;
-  for(;;)
-   {
-#ifdef SMALLHEAP
-     if((size_t)*filled>=(size_t)(*filled+blksiz))
-	lcking|=lck_MEMORY,nomemerr(*filled);
-#endif				       /* dynamically adjust the buffer size */
-		       /* use the real realloc so that we can retry failures */
-     while(EXPBLKSIZ&&(np=0,blksiz>BLKSIZ)&&!(np=frealloc(bf,*filled+blksiz)))
-	blksiz>>=1;				  /* try a smaller increment */
-     bf=EXPBLKSIZ&&np?np:trealloc(bf,(size_t)(*filled+blksiz));	 /* last try */
-jumpback:;
-     ;{ int got,left=blksiz;
-	do
-	   if(0>=(got=rread(STDIN,bf+*filled,left)))		/* read mail */
-	      goto eoffound;
-	while(*filled+=got,left-=got);		/* change listed buffer size */
-      }
-     if(EXPBLKSIZ&&shift)				 /* room for growth? */
-      { int newbs=blksiz;newbs<<=shift--;	/* capped exponential growth */
-	if(blksiz<newbs)				  /* no overflowing? */
-	   blksiz=newbs;				    /* yes, take me! */
-      }
-   }
-eoffound:
+static char*read_read(p,left,data)char*p;long left;void*data;
+{ long got;
+  do
+     if(0>=(got=rread(STDIN,p,left)))				/* read mail */
+	return p;
+  while(p+=got,left-=got);			/* change listed buffer size */
+  return 0;
+}
+
+static int read_cleanup(mb,filledp,origfilled,data)memblk*mb;
+ long*filledp,origfilled;void*data;
+{ long oldfilled= *(long*)data;
   if(pidchild>0)
    { if(PRDB>=0)
       { getstdin(PRDB);			       /* filter ready, get backpipe */
 	if(1==rread(STDIN,buf,1))		      /* backup pipe closed? */
-	 { bf=trealloc(bf,(size_t)((*filled=oldsize+1)+blksiz));
-	   bf[oldsize]= *buf;
+	 { resizeblock(mb,*filledp=oldfilled,0);
+	   mb->p[origfilled]= *buf;
+	   *filledp=origfilled++;
 	   PRDB= -1;pwait=2;		      /* break loop, definitely reap */
-	   goto jumpback;		       /* filter goofed, rescue data */
+	   return 1;			       /* filter goofed, rescue data */
 	 }
       }
      if(pwait)
 	pipw=waitfor(pidchild);		      /* reap your child in any case */
    }
   pidchild=0;					/* child must be gone by now */
-  return (np=frealloc(bf,*filled+1))?np:bf;   /* minimise+1 for housekeeping */
+  return 0;
+}
+
+char*readdyn(mb,filled,oldfilled)memblk*const mb;long*const filled,oldfilled;
+{ return read2blk(mb,filled,&read_read,&read_cleanup,&oldfilled);
 }
 
 char*fromprog(name,dest,max)char*name;char*const dest;size_t max;
@@ -299,7 +300,7 @@ char*fromprog(name,dest,max)char*name;char*const dest;size_t max;
      rclose(PWRI);rclose(PRDO);
      if(forkerr(pidfilt,name))
 	rclose(PWRO),stermchild();
-     dump(PWRO,ft_PIPE,themail,filled);waitfor(pidfilt);exit(lexitcode);
+     dump(PWRO,ft_PIPE,themail.p,filled);waitfor(pidfilt);exit(lexitcode);
    }
   rclose(PWRI);p=dest;
   if(!forkerr(pidchild,name))
@@ -320,19 +321,12 @@ char*fromprog(name,dest,max)char*name;char*const dest;size_t max;
 
 void exectrap(tp)const char*const tp;
 { int forceret;
-  ;{ char*p;
-     if(setxit&&(p=getenv(exitcode)))		 /* user specified exitcode? */
-      { if((forceret=renvint(-2L,p))>=0)	     /* yes, is it positive? */
-	   retval=forceret;				 /* then override it */
-      }
-     else if(*tp)		 /* no EXITCODE set, TRAP found, provide one */
-      { strcpy(p=buf2,exitcode);*(p+=STRLEN(exitcode))='=';
-	ultstr(0,(unsigned long)retval,p+1);sputenv(buf2);forceret= -1;
-      }
-   }
+  forceret=setexitcode(*tp);		      /* whether TRAP is set matters */
   if(*tp)
    { int poutfd[2];
-     metaparse(tp);concon('\n');rpipe(poutfd);inittmout(buf);
+     rawnonl=0;					 /* force a trailing newline */
+     metaparse(tp);concon('\n');			     /* expand $TRAP */
+     rpipe(poutfd);inittmout(buf);
      if(!(pidchild=sfork()))	     /* connect stdout to stderr before exec */
       { rclose(PWRO);getstdin(PRDO);rclose(STDOUT);rdup(STDERR);
 	callnewprog(buf);			      /* trap your heart out */
@@ -340,7 +334,7 @@ void exectrap(tp)const char*const tp;
      rclose(PRDO);					     /* neat & clean */
      if(!forkerr(pidchild,buf))
       { int newret;
-	dump(PWRO,ft_PIPE,themail,filled);    /* try and shove down the mail */
+	dump(PWRO,ft_PIPE,themail.p,filled);  /* try and shove down the mail */
 	if((newret=waitfor(pidchild))!=EXIT_SUCCESS&&forceret==-2)
 	   retval=newret;		       /* supersede the return value */
 	pidchild=0;
